package ca.ntro.ntro_app_fx_impl;

import java.io.IOException;

import ca.ntro.app.Ntro;
import ca.ntro.app.NtroServerFx;
import ca.ntro.app.common.ServerRegistrarJdkImpl;
import ca.ntro.app.common.WebSocketServerNtro;
import ca.ntro.core.NtroCore;
import ca.ntro.core.services.ExitHandler;
import ca.ntro.ntro_app_fx_impl.NtroImpl;
import ca.ntro.ntro_app_fx_impl.backend.BackendRegistrarNtro;
import ca.ntro.ntro_app_fx_impl.messages.MessageRegistrarNtro;
import ca.ntro.ntro_app_fx_impl.models.ModelRegistrarImpl;
import ca.ntro.ntro_core_impl.NtroCoreImpl;
import javafx.application.Application;
import javafx.stage.Stage;

public class ServerWrapperFx extends Application implements ExitHandler {
    
    public static Class<? extends NtroServerFx> serverClass;
    
    private NtroServerFx serverSpec;
    private WebSocketServerNtro webSocketServer;

    private MessageRegistrarNtro   messageRegistrar  = new MessageRegistrarNtro();
    private ModelRegistrarImpl     modelRegistrar    = new ModelRegistrarImpl();
    private BackendRegistrarNtro   backendRegistrar  = new BackendRegistrarNtro(modelRegistrar, messageRegistrar);
    private ServerRegistrarJdkImpl serverRegistrar   = new ServerRegistrarJdkImpl();

    public NtroServerFx getServerSpec() {
        return serverSpec;
    }

    public void setServerSpec(NtroServerFx serverSpec) {
        this.serverSpec = serverSpec;
    }

    public WebSocketServerNtro getWebSocketServer() {
        return webSocketServer;
    }

    public void setWebSocketServer(WebSocketServerNtro webSocketServer) {
        this.webSocketServer = webSocketServer;
    }

    public ServerWrapperFx() {
    }

    public ServerWrapperFx(NtroServerFx serverSpec) {
        setServerSpec(serverSpec);
    }

    @Override
    public void start(Stage primaryStage) throws Exception {
		NtroCoreImpl.registerAppExitHandler(this);

        Runtime.getRuntime().addShutdownHook(new Thread() {
        	@Override
        	public void run() {
        		Ntro.exit();
			}
        });

        NtroCore.logger().run();

        new Thread() {
            
            @Override
            public void run() {

                NtroCore.logger().info("App running. Press Enter here to close App");

                try {

                    System.in.read();
                    NtroCore.exit();

                } catch (IOException e) {

                    e.printStackTrace();
                }
            }
        }.start();
        
        try {
        
			
			serverSpec = NtroCore.factory().newInstance(serverClass);
			
			serverSpec.registerMessages(messageRegistrar);
			serverSpec.registerModels(modelRegistrar);
			serverSpec.registerBackend(backendRegistrar);
			serverSpec.registerServer(serverRegistrar);

			System.out.println("\n\n\n");
			NtroCore.logger().info("Ntro version 0.4");
			NtroCore.logger().info("Locale: '" + Ntro.currentLocale() + "'");
			
			backendRegistrar.prepareToExecuteTasks();
			
			NtroCore.threads().runOnMainThread(() -> {
				backendRegistrar.executeTasks();
			});

			webSocketServer = new WebSocketServerNtro(serverRegistrar.getServerName(), serverRegistrar.getPort());
			webSocketServer.start();
			
        } catch(Throwable t) {
        	
        	NtroCoreImpl.logger().fatal(t);
        	
        }
        
    }

    @Override
    public void onExit() {

        NtroCore.logger().info("Writing JSON files");
        NtroImpl.models().writeModelFiles();

        NtroCore.logger().info("Generating graphs");
        NtroImpl.models().writeGraphs();

        if(!backendRegistrar.isRemoteBackend()) {
            backendRegistrar.writeGraph();
        }

    	try {
    		
    		if(webSocketServer != null) {
				webSocketServer.stop();
    		}

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
    }

}
