package ca.ntro.ntro_core_impl.options;

import java.nio.file.Paths;

import ca.ntro.core.options.CoreOptions;
import ca.ntro.core.options.CoreOptionsWriter;
import ca.ntro.ntro_core_impl.NtroCoreImpl;

public class CoreOptionsImpl implements CoreOptions, CoreOptionsWriter {
	
	private boolean displayWarnings = true;
	private boolean displayTasks    = true;

	private String projectPath   = "${userDir}";
	private String storagePath   = "${projectPath}/_storage";
	private String modelsPath    = "${storagePath}/models";
	private String tmpPath       = "${storagePath}/tmp";
	private String sessionsPath  = "${storagePath}/sessions";
	private String optionsPath   = "${storagePath}/options";
	private String graphsPath    = "${storagePath}/graphs";
	private String resourcesPath = "${projectPath}/src/main/resources";
	
	private transient String userDirImpl       = null;
	private transient String projectPathImpl   = null;
	private transient String storagePathImpl   = null;
	private transient String modelsPathImpl    = null;
	private transient String tmpPathImpl       = null;
	private transient String sessionsPathImpl  = null;
	private transient String optionsPathImpl   = null;
	private transient String graphsPathImpl    = null;
	private transient String resourcesPathImpl = null;

	@Override
	public void projectPath(String projectPath) {
		this.projectPath = projectPath;
		resolvePaths();
	}

	@Override
	public void resourcesPath(String resourcesPath) {
		this.resourcesPath = resourcesPath;
		resolvePaths();
	}

	@Override
	public void displayWarnings(boolean displayWarnings) {
		this.displayWarnings = displayWarnings;
	}

	@Override
	public void displayTasks(boolean displayTasks) {
		this.displayTasks = displayTasks;
		
	}

	@Override
	public boolean displayWarnings() {
		return displayWarnings;
	}

	@Override
	public boolean displayTasks() {
		return displayTasks;
	}

	@Override
	public String projectPath() {
		return projectPathImpl;
	}

	@Override
	public String resourcesPath() {
		return resourcesPathImpl;
	}

	@Override
	public String storagePath() {
		return storagePathImpl;
	}

	@Override
	public String modelsPath() {
		return modelsPathImpl;
	}

	@Override
	public String tmpPath() {
		return tmpPathImpl;
	}

	@Override
	public String sessionsPath() {
		return sessionsPathImpl;
	}

	@Override
	public String graphsPath() {
		return graphsPathImpl;
	}

	@Override
	public String optionsPath() {
		return optionsPathImpl;
	}
	
	/**
	 * userDir
	 * projectPath
	 * storagePath
	**/
	private String resolvePath(String path) {
	
		String resolved = path;

		resolved = resolved.replaceAll("\\$\\{userDir\\}", userDirImpl);
		resolved = resolved.replaceAll("\\$\\{projectPath\\}", projectPathImpl);
		resolved = resolved.replaceAll("\\$\\{storagePath\\}", storagePathImpl);
		
		return resolved;
		
	}
	
	public void resolvePaths() {
		
		userDirImpl       = System.getProperty("user.dir").replaceAll("\\\\","/");
		projectPathImpl   = resolvePath(projectPath);
		storagePathImpl   = resolvePath(storagePath);
		modelsPathImpl    = resolvePath(modelsPath);
		tmpPathImpl       = resolvePath(tmpPath);
		sessionsPathImpl  = resolvePath(sessionsPath);
		optionsPathImpl   = resolvePath(optionsPath);
		graphsPathImpl    = resolvePath(graphsPath);
		resourcesPathImpl = resolvePath(resourcesPath);
	}

}
