package pong.frontal.taches;

import ca.ntro.app.frontend.Tick;
import ca.ntro.app.modified.Modified;
import ca.ntro.app.tasks.frontend.FrontendTasks;
import pong.commun.modeles.ModelePartie;
import pong.frontal.donnees.DonneesVuePartie;
import pong.frontal.evenements.EvtActionJoueur;
import pong.frontal.evenements.EvtClicSouris;
import pong.frontal.vues.VuePartie;

import static ca.ntro.app.tasks.frontend.FrontendTasks.*;


public class AfficherPartie {
	
	public static void creerTaches(FrontendTasks tasks) {
		
		tasks.taskGroup("AfficherPartie")

		     .waitsFor("PremierAffichage")
		     
		     .contains(subTasks -> {

				reagirActionJoueur(subTasks);

				reagirClicSouris(subTasks);

				prochaineImagePartie(subTasks);

				afficherInfoJoueurs(subTasks);

		     });
	}

	private static void afficherInfoJoueurs(FrontendTasks subTasks) {
		subTasks.task("afficherInfoJoueurs")

		        .waitsFor(created(VuePartie.class))

		        .waitsFor(modified(ModelePartie.class))
		        
		        .executes(inputs -> {
		        	
		        	VuePartie              vuePartie      = inputs.get(created(VuePartie.class));
		        	Modified<ModelePartie> modifiedPartie = inputs.get(modified(ModelePartie.class));
		        	
		        	ModelePartie partie = modifiedPartie.currentValue();
		        	
		        	partie.afficherSur(vuePartie);
		        	
		        	vuePartie.requestFocus();
		        	
		        });
	}

	private static void prochaineImagePartie(FrontendTasks subTasks) {

		subTasks.task("prochaineImagePartie")

		        .waitsFor(clock().nextTick())
		        
		        .waitsFor(created(VuePartie.class))

		        .waitsFor(created(DonneesVuePartie.class))
		
		        .executes(inputs -> {
		        	
					Tick             tick             = inputs.get(clock().nextTick());
					DonneesVuePartie donneesVuePartie = inputs.get(created(DonneesVuePartie.class));
					VuePartie        vuePartie        = inputs.get(created(VuePartie.class));
					
					donneesVuePartie.reagirTempsQuiPasse(tick.elapsedTime());
					donneesVuePartie.afficherSur(vuePartie);

		        });
	}


	private static void reagirActionJoueur(FrontendTasks subTasks) {

		subTasks.task("reagirActionJoueur")

		        .waitsFor(event(EvtActionJoueur.class))

		        .waitsFor(created(DonneesVuePartie.class))

		        .executes(inputs -> {
		        	
		        	DonneesVuePartie       donneesVuePartie = inputs.get(created(DonneesVuePartie.class));
		        	EvtActionJoueur        evtActionJoueur  = inputs.get(event(EvtActionJoueur.class));
		        	
		        	evtActionJoueur.appliquerA(donneesVuePartie);

		        });
	}

	private static void reagirClicSouris(FrontendTasks subTasks) {

		subTasks.task("reagirClicSouris")

		        .waitsFor(event(EvtClicSouris.class))

		        .waitsFor(created(DonneesVuePartie.class))
		        
		        .executes(inputs -> {
		        	
		        	DonneesVuePartie donneesVuePartie = inputs.get(created(DonneesVuePartie.class));
		        	EvtClicSouris    evtClicSouris    = inputs.get(event(EvtClicSouris.class));

		        	evtClicSouris.appliquerA(donneesVuePartie);

		        });
	}
}
