package pong.frontal.vues;

import ca.ntro.app.Ntro;
import ca.ntro.app.frontend.ViewFx;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import pong.commun.enums.Action;
import pong.commun.enums.Position;
import pong.commun.monde2d.MondePong2d;
import pong.frontal.controles.CanvasPartie;
import pong.frontal.evenements.EvtClicSouris;
import pong.frontal.utils.ActionsActivesParPosition;

public class VuePartie extends ViewFx {

	@FXML
	private Pane conteneurVuePartie;
	
	@FXML
	private CanvasPartie canvasPartie;
	
	@FXML
	private Label labelNomPremierJoueur;

	@FXML
	private Label labelNomDeuxiemeJoueur;

	@FXML
	private Label labelScorePremierJoueur;

	@FXML
	private Label labelScoreDeuxiemeJoueur;
	
	private ActionsActivesParPosition actionsActives = new ActionsActivesParPosition();

	@Override
	public void initialize() {
		Ntro.assertNotNull(canvasPartie);

		Ntro.assertNotNull(labelNomPremierJoueur);
		Ntro.assertNotNull(labelNomDeuxiemeJoueur);

		Ntro.assertNotNull(labelScorePremierJoueur);
		Ntro.assertNotNull(labelScoreDeuxiemeJoueur);
		
		Ntro.assertNotNull(conteneurVuePartie);
		
		installerEvtClicSouris();
		installerEvtActionJoueur();

	}
	
	public void requestFocus() {
		conteneurVuePartie.requestFocus();
	}

	private void installerEvtClicSouris() {

		canvasPartie.onMouseEvent(mouseEventNtro -> {
			
			if(mouseEventNtro.mouseEventFx().getEventType().equals(MouseEvent.MOUSE_CLICKED)) {

				Ntro.newEvent(EvtClicSouris.class)
					.setMouseEvent(mouseEventNtro)
					.trigger();
			}
		});
	}

	private void installerEvtActionJoueur() {
		conteneurVuePartie.addEventFilter(KeyEvent.KEY_PRESSED, evtFx -> {
			
			Ntro.logger().info("touche appuyée: " + evtFx.getCode());

			Position position = positionPourTouche(evtFx.getCode());
			Action action = actionPourTouche(evtFx.getCode());
			
			actionsActives.activer(position, action);
		});

		conteneurVuePartie.addEventFilter(KeyEvent.KEY_RELEASED, evtFx -> {

			Position position = positionPourTouche(evtFx.getCode());
			Action action = actionPourTouche(evtFx.getCode());
			
			actionsActives.desactiver(position, action);
		});
	}

	private Position positionPourTouche(KeyCode code) {
		Position position = null;
		
		position = positionPourToucheApp(code);
			
		return position;
	}

	private Position positionPourToucheApp(KeyCode code) {
		Position position = Position.DROITE;

		if(code.equals(KeyCode.W)
				|| code.equals(KeyCode.S)) {
			
			position = Position.GAUCHE;
		}

		return position;
	}


	private Action actionPourTouche(KeyCode code) {
		Action action = Action.ARRETER;
		
		if(code.equals(KeyCode.W) || code.equals(KeyCode.UP)) {
			
			action = Action.MONTER;
			
		} else if(code.equals(KeyCode.S) || code.equals(KeyCode.DOWN)) {

			action = Action.DESCENDRE;

		}
		
		return action;
	}

	public void viderCanvas() {
		canvasPartie.clearCanvas();
	}

	public void afficherPong2d(MondePong2d mondePong2d) {
		mondePong2d.drawOn(canvasPartie);
	}

	public void afficherImagesParSeconde(String imagesParSeconde) {
		canvasPartie.afficherFps(imagesParSeconde);
	}


	public void afficherPrenomJoueurGauche(String prenom) {
		labelNomPremierJoueur.setText(prenom);
	}

	public void afficherPrenomJoueurDroit(String prenom) {
		labelNomDeuxiemeJoueur.setText(prenom);
	}

	public void afficherScoreJoueurGauche(int score) {
		labelScorePremierJoueur.setText(String.valueOf(score));
	}

	public void afficherScoreJoueurDroit(int score) {
		labelScoreDeuxiemeJoueur.setText(String.valueOf(score));
	}

}
